import base64
import requests

from urllib.parse import urljoin
from .base_client import BaseClient


class WebhookClient(BaseClient):
    def public_key(self):
        query_url = urljoin(self.base_url, 'webhooks/public_key')

        api_response = requests.get(query_url, headers=self.headers())
        coded_response = self.handle_response(api_response).text

        return base64.b64decode(coded_response)

    def headers(self):
        bearer = "Bearer " + self.api_key
        headers = {'Authorization': bearer}

        return headers



from pydantic import BaseModel, Field
from typing import Optional


class AppliedCoupon(BaseModel):
    customer_id: str
    coupon_code: str
    amount_cents: Optional[int]
    amount_currency: Optional[str]


class AppliedCouponResponse(BaseModel):
    lago_id: str
    lago_coupon_id: str
    coupon_code: str
    customer_id: str
    lago_customer_id: str
    amount_cents: str
    amount_currency: str
    expiration_date: Optional[str]
    created_at: str
    terminated_at: Optional[str]



from pydantic import BaseModel, Field
from typing import Optional


class AppliedAddOn(BaseModel):
    customer_id: str
    add_on_code: str
    amount_cents: Optional[int]
    amount_currency: Optional[str]


class AppliedAddOnResponse(BaseModel):
    lago_id: str
    lago_add_on_id: str
    add_on_code: str
    customer_id: str
    lago_customer_id: str
    amount_cents: str
    amount_currency: str
    created_at: str



from tests.test_event_client import TestEventClient
from tests.test_subscription_client import TestSubscriptionClient
from tests.test_customer_client import TestCustomerClient
from tests.test_applied_coupon_client import TestAppliedCouponClient
from tests.test_applied_add_on_client import TestAppliedAddOnClient
from tests.test_webhook_client import TestWebhookClient
from tests.test_client import TestClient



from pydantic import BaseModel, Field
from typing import Optional


class Subscription(BaseModel):
    plan_code: str
    customer_id: str


class SubscriptionResponse(BaseModel):
    lago_id: str
    lago_customer_id: Optional[str]
    customer_id: Optional[str]
    canceled_at: Optional[str]
    created_at: Optional[str]
    plan_code: Optional[str]
    started_at: Optional[str]
    status: Optional[str]
    terminated_at: Optional[str]



from .base_client import BaseClient
from lago_python_client.models.subscription import SubscriptionResponse
from typing import Dict


class SubscriptionClient(BaseClient):
    def api_resource(self):
        return 'subscriptions'

    def root_name(self):
        return 'subscription'

    def prepare_response(self, data: Dict):
        return SubscriptionResponse.parse_obj(data)



import unittest
import requests_mock

from lago_python_client.client import Client
from lago_python_client.clients.base_client import LagoApiError


class TestWebhookClient(unittest.TestCase):
    def test_valid_public_key_request(self):
        client = Client(api_key='886fe239-927d-4072-ab72-6dd345e8dd0d')

        with requests_mock.Mocker() as m:
            m.register_uri('GET', 'https://api.getlago.com/api/v1/webhooks/public_key', text='a2V5')
            response = client.webhooks().public_key()

        self.assertEqual(response, b'key')

    def test_invalid_public_key_request(self):
        client = Client(api_key='invalid')

        with requests_mock.Mocker() as m:
            m.register_uri('GET', 'https://api.getlago.com/api/v1/webhooks/public_key', status_code=401, text='')

            with self.assertRaises(LagoApiError):
                client.webhooks().public_key()


if __name__ == '__main__':
    unittest.main()



import requests
import json

from pydantic import BaseModel
from requests import Response
from typing import Dict
from urllib.parse import urljoin


class BaseClient:
    RESPONSE_SUCCESS_CODES = [200, 201, 202, 204]

    def __init__(self, base_url: str, api_key: str):
        self.base_url = base_url
        self.api_key = api_key

    def create(self, input_object: BaseModel):
        query_url = urljoin(self.base_url, self.api_resource())
        query_parameters = {
            self.root_name(): input_object.dict()
        }
        data = json.dumps(query_parameters)
        api_response = requests.post(query_url, data=data, headers=self.headers())
        data = self.handle_response(api_response)

        if data is None:
            return True
        else:
            return self.prepare_response(data.json().get(self.root_name()))

    def delete(self, params: Dict):
        query_url = urljoin(self.base_url, self.api_resource())
        data = json.dumps(params)
        api_response = requests.delete(query_url, data=data, headers=self.headers())
        data = self.handle_response(api_response).json().get(self.root_name())

        return self.prepare_response(data)

    def headers(self):
        bearer = "Bearer " + self.api_key
        headers = {
            'Content-type': 'application/json',
            'Authorization': bearer,
            'User-agent': 'Lago Python v0.1.3'
        }

        return headers

    def handle_response(self, response: Response):
        if response.status_code in BaseClient.RESPONSE_SUCCESS_CODES:
            if response.text:
                return response
            else:
                return None
        else:
            raise LagoApiError(
                "URI: %s. Status code: %s. Response: %s." % (
                    response.request.url, response.status_code, response.text)
            )


class LagoApiError(Exception):
    ...



from lago_python_client.clients.applied_add_on_client import AppliedAddOnClient
from lago_python_client.clients.applied_coupon_client import AppliedCouponClient
from lago_python_client.clients.event_client import EventClient
from lago_python_client.clients.subscription_client import SubscriptionClient
from lago_python_client.clients.customer_client import CustomerClient
from lago_python_client.clients.webhook_client import WebhookClient



from pydantic import BaseModel, Field
from typing import Optional


class Event(BaseModel):
    transaction_id: str
    customer_id: str
    code: str
    timestamp: Optional[int]
    properties: Optional[dict]



import unittest
import requests_mock
import os

from lago_python_client.client import Client
from lago_python_client.models import Customer
from lago_python_client.clients.base_client import LagoApiError


def create_customer():
    return Customer(customer_id='5eb02857-a71e-4ea2-bcf9-57d3a41bc6ba', name='Gavin Belson')


def mock_response():
    this_dir = os.path.dirname(os.path.abspath(__file__))
    my_data_path = os.path.join(this_dir, 'fixtures/customer.json')

    with open(my_data_path, 'r') as subscription_response:
        return subscription_response.read()


class TestCustomerClient(unittest.TestCase):
    def test_valid_create_customers_request(self):
        client = Client(api_key='886fe239-927d-4072-ab72-6dd345e8dd0d')

        with requests_mock.Mocker() as m:
            m.register_uri('POST', 'https://api.getlago.com/api/v1/customers', text=mock_response())
            response = client.customers().create(create_customer())

        self.assertEqual(response.customer_id, '5eb02857-a71e-4ea2-bcf9-57d3a41bc6ba')
        self.assertEqual(response.name, 'Gavin Belson')
        self.assertEqual(response.email, 'dinesh@piedpiper.test')

    def test_invalid_create_customers_request(self):
        client = Client(api_key='invalid')

        with requests_mock.Mocker() as m:
            m.register_uri('POST', 'https://api.getlago.com/api/v1/customers', status_code=401, text='')

            with self.assertRaises(LagoApiError):
                client.customers().create(create_customer())


if __name__ == '__main__':
    unittest.main()



from lago_python_client.clients.applied_add_on_client import AppliedAddOnClient
from lago_python_client.clients.applied_coupon_client import AppliedCouponClient
from lago_python_client.clients.subscription_client import SubscriptionClient
from lago_python_client.clients.customer_client import CustomerClient
from lago_python_client.clients.event_client import EventClient
from lago_python_client.clients.webhook_client import WebhookClient
from urllib.parse import urljoin


class Client:
    BASE_URL = 'https://api.getlago.com/'
    API_PATH = 'api/v1/'

    def __init__(self, api_key: str = None, api_url: str = None):
        self.api_key = api_key
        self.api_url = api_url

    def base_api_url(self):
        if self.api_url is None:
            return urljoin(Client.BASE_URL, Client.API_PATH)
        else:
            return urljoin(self.api_url, Client.API_PATH)

    def events(self):
        return EventClient(self.base_api_url(), self.api_key)

    def subscriptions(self):
        return SubscriptionClient(self.base_api_url(), self.api_key)

    def customers(self):
        return CustomerClient(self.base_api_url(), self.api_key)

    def applied_coupons(self):
        return AppliedCouponClient(self.base_api_url(), self.api_key)

    def applied_add_ons(self):
        return AppliedAddOnClient(self.base_api_url(), self.api_key)

    def webhooks(self):
        return WebhookClient(self.base_api_url(), self.api_key)



import unittest
import requests_mock

from lago_python_client.client import Client
from lago_python_client.models import Event
from lago_python_client.clients.base_client import LagoApiError


def create_event():
    return Event(customer_id='5eb02857-a71e-4ea2-bcf9-57d8885990ba', code='123', transaction_id='123')


class TestEventClient(unittest.TestCase):
    def test_valid_create_events_request(self):
        client = Client(api_key='886fe239-927d-4072-ab72-6dd345e8dd0d')

        with requests_mock.Mocker() as m:
            m.register_uri('POST', 'https://api.getlago.com/api/v1/events', text='')
            response = client.events().create(create_event())

        self.assertEqual(response, True)

    def test_invalid_create_events_request(self):
        client = Client(api_key='invalid')

        with requests_mock.Mocker() as m:
            m.register_uri('POST', 'https://api.getlago.com/api/v1/events', status_code=401, text='')

            with self.assertRaises(LagoApiError):
                client.events().create(create_event())


if __name__ == '__main__':
    unittest.main()



from lago_python_client.models.applied_add_on import AppliedAddOn
from lago_python_client.models.applied_coupon import AppliedCoupon
from lago_python_client.models.event import Event
from lago_python_client.models.customer import Customer
from lago_python_client.models.subscription import Subscription



from .base_client import BaseClient
from lago_python_client.models.customer import CustomerResponse
from typing import Dict


class CustomerClient(BaseClient):
    def api_resource(self):
        return 'customers'

    def root_name(self):
        return 'customer'

    def prepare_response(self, data: Dict):
        return CustomerResponse.parse_obj(data)
